------------------------------------------------------------------------
--  LcdDemo.vhd -- Pabi Waledi game LcdDemo VHDL file
------------------------------------------------------------------------
--  Author : Mircea Dabacan
--           Copyright 2004 Digilent, Inc.
------------------------------------------------------------------------
-- Software version: Xilinx ISE 6.2.03i
--                   WebPack
------------------------------------------------------------------------
-- This is the LcdDemo source file for the Tennis (Pabi_Waledi) project, 
-- provided by the Digilent Reference Project Library.

-- This file contains structural high level of the Tennis project.
-- It is very similar to a schematic description 
-- (showing the components and the connections between them).
-- The user can compare it to the LcdDemo.sch in the schematic variant of 
-- the project (when available).

-- For details about the game and the project components, see the 
-- component source files.

------------------------------------------------------------------------
-- Revision History:
--   12/21/2004(MirceaD): created
--   03/26/2005(MirceaD): formatted
------------------------------------------------------------------------
														   
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_arith.all;  -- defines the conv_std_logic_vector 
                               -- function, used bellow.
library work;
use work.LCD_pkg.all;

entity LcdDemo is
  Port (ck : in std_logic;               -- 50MHz board clock
    res_btn: in std_logic;               -- btn1 on the LcdDemo board 
    db : inout std_logic_vector(7 downto 0);    -- data bus
--    DBadr : out std_logic_vector (5 downto 0);  -- data bus address
--    cs, oe, we: out std_logic;          -- data bus chip select signals
    lcd_E: out std_logic;               -- LCD enable (active high)
    lcd_RS: OUT std_logic;    -- LCD Register Select ('1' - data, 
                              --                      '0' - instruction)
    lcd_RW: OUT std_logic;    -- LCD Read/Write ('1'- read, '0'- write)
--    sSegCk: out std_logic);   -- clock for CPLD chip

led_state: inout std_logic_vector (7 downto 0);    
       --  16 LEDs, LD0 (right) to LD15 (left) 
btn: in std_logic_vector (4 downto 0));
       -- keyboard signals (not all of them used)
--StrataFlashCS: out std_logic:='1');


end LcdDemo;

architecture structural of LcdDemo is
-- divided clock signals:
signal ck3MHz, ck1KHz, ck24Hz : std_logic; -- system clock divided by 16

-- signals for db_cntrl <-> LCD_mess communication:
signal LCD_ack : std_logic;   -- LCD cycle acknowledge
signal LCD_req : std_logic;   -- LCD cycle request
signal lcd_RS_req: std_logic; -- LCD Register Select request('1' - data, 
                              --                      '0' - instruction)
signal lcd_RW_req: std_logic;-- LCD Read/Write request ('1' - read, 
                              --                         '0' - write)
signal LCD_write_byte:std_logic_vector(7 downto 0);--byte to send to LCD
signal LCD_read_byte:std_logic_vector (7 downto 0);--byte read from LCD

-- signals for user_application <-> db_cntrl communication:
--signal led_state: std_logic_vector (15 downto 0);    
       --  16 LEDs, LD0 (right) to LD15 (left) 
--signal btn: std_logic_vector (14 downto 0);
       -- keyboard signals (not all of them used)

-- signals for user_application <-> LCD_mess communication:
signal LCD_message :string(1 to 41);
signal lcd_config_string : config_string_type (1 to 6);
signal LCD_mes_req, LCD_mes_ack : std_logic;

signal start_game : std_logic;                -- start game command 
signal turbo_left,turbo_right, hit_left, hit_right: std_logic;  
       -- play commands (keyboard buttons)
signal score_left, score_right: integer range 0 to 9; 
signal disp7s: std_logic_vector(15 downto 0);
signal reset :std_logic;    

component BUFG      -- bufg declared for btn1 on the LcdDemo board
   port (I: in std_logic; O: out std_logic);
end component;   
 
component ck_div    -- divides the 50MHz clock by 1250
  Port (ck:in std_logic;  -- 50MHz board clock
    ck3MHz:out std_logic; -- clock for dbIOxCtrl, Lcd_Mess_Ctrl
    ck1KHz:out std_logic; -- clock for DIOx CPLD chip(7 segment display)
    ck24Hz:out std_logic);-- clock for Pabi_WalediFSM
end component;   

component GraphicCtrl    -- play state machine
  port (ck, RESET: in std_logic; -- general signals
  -- keyboard signals
    hit_left,hit_right,start_game,turbo_left,turbo_right: in std_logic;
  -- LEDs
    led_state: inout std_logic_vector (7 downto 0);
               -- feedback requires inout
  -- Score - interface to dbIOxCtrl
    score_left, score_right: inout integer range 0 to 9;  
                             -- feedback requires inout
  -- Interface to LcdMessCtrl signals
    lcd_config_string : out config_string_type(1 to 6);  
    -- LCD configuration string   
    lcd_message : out string(1 to 41);               -- LCD text message
    lcd_mes_req : inout std_logic;   -- request to send a message to LCD  
    lcd_mes_ack : in std_logic);  -- dbIOxCtrl begun sending the message
end component;

  component LcdSwCtrl         -- IO2 bus controller
  port(
    ck : in STD_LOGIC;                    -- BDstate machine clock
    reset : in std_logic;                 -- Reset signal
-- Digilent IO2 bus signals:
    IO2_db   : inout std_logic_vector (7 downto 0); -- data bus
--    IO2_adr  : out std_logic_vector (5 downto 0);   -- address
--    IO2_CS, IO2_OE, IO2_WE :out std_logic;          -- control signals
-- Digilent IO2 resources images:
--    disp7s  : std_logic_vector (15 downto 0);    
    -- four digit hexadecimal number for the 7 seg display
--    LED_state : in STD_LOGIC_VECTOR (15 downto 0);
    --  16 LEDs, LD0 (left) to LD15 (right) 
--   btn : inout std_logic_vector (14 downto 0);   -- DIO2 keypad buttons
--    sw : inout std_logic_vector (7 downto 0);     -- DIO2 swithces
-- Digilent IO2 Lcd control signals:
    Lcd_E : out std_logic;    -- Lcd enable (active high)
    Lcd_RS : out std_logic;   -- Lcd Register Select ('1' - data, 
                              --                      '0' - instruction)
    Lcd_RW : out std_logic;   -- Lcd Read/Write ('1'- read, '0'- write)
-- Handshake signals dealing with Lcd interface
    Lcd_req: in std_logic;                  -- Lcd cycle request
    Lcd_RS_req: in std_logic; -- Lcd Register Select request('1' - data,
                              --                      '0' - instruction)
    Lcd_RW_req: in std_logic; -- Lcd Read/Write request ('1' - read, 
                              --                         '0' - write)
    Lcd_ack: inout std_logic; -- Lcd cycle acknowledge
-- Lcd images:
    Lcd_write_byte: in std_logic_vector (7 downto 0);  
    -- the byte to send to Lcd
    Lcd_read_byte: out std_logic_vector (7 downto 0)  
    -- the byte read from Lcd
      );
   end component;

  component LCD_mess_Ctrl      -- LCD Message controller
  Port ( 
-- General purpose signals
    ck : in std_logic;                    -- 3.125MHz clock
    reset: in std_logic;                  -- btn1 on the LcdDemo board 
--  User application interface signals
    lcd_config_string: in config_string_type;  
    -- the configuration string to be sent to the LCD
    LCD_message:in string;     -- the text string to be sent to the LCD
    LCD_mes_req: in std_logic; 
    -- the user application request to send a message
    LCD_mes_ack: out std_logic;
    -- the acknowledge signal generated by the Lcd_Mess_Ctrl
-- dbIOxCtrl component interface signals 
    lcd_RS_req: out std_logic;-- LCD Register Select request('1' - data,
                              --                      '0' - instruction)
    lcd_RW_req: out std_logic;-- LCD Read/Write request ('1' - read, 
                              --                         '0' - write)
    LCD_write_byte: out std_logic_vector (7 downto 0);  
    -- the byte to send to LCD
    LCD_req: out std_logic;              -- LCD cycle request
    LCD_ack: in std_logic                -- LCD cycle acknowledge
    );                
  end component;

begin
-- actual architecture begin


--StrataFlashCS <= '1';


--Note: When the BTN1 on the LcdDemo board is wired (hardware board design)
  -- to a GCKx FPGA pin, it requires a BUFG to be assigned for the 
  -- corresponding signal (res_btn in this project). 
  -- For this kind of Digilent boards (Digilab II  XL), a BUFG needs to
  -- be declared (see the declaration segment above, between 
  -- "architecture" and "begin" keywords) and instantiated (see the line
  -- below the note).
  -- The signal assignment line below the BUFG instantiation is neither
  -- necessary nor accepted by the compiler.

  -- When the BTN1 on the LcdDemo board is wired (hardware board design) to
  -- a general input/output FPGA pin, a BUFG is neither required nor 
  -- allowed (res_btn in this project). 
  -- For this kind of Digilent boards (Digilab 2FT, 2SB), both BUFG 
  -- declaration and instantiation must be removed (deleted or 
  -- commented). Instead, the signal assignment line below the BUFG 
  -- instantiation is necessary.

-- BTN1: BUFG port map (I => not res_btn, O => reset);    
         -- bufg instantiated for btn1 on the LcdDemo board
reset <= res_btn;


  Inst_ck_div: ck_div port map(    -- divides the 50MHz system clock 
    ck      => ck,
    ck3MHz  => ck3MHz,   -- clock for dbIOxCtrl, Lcd_Meaa_Ctrl
    ck1KHz  => ck1KHz,   -- clock for DIO5 CPLD chip (7 segment display)
    ck24Hz  => ck24Hz    -- clock for Pabi_WalediFSM
    );

  Inst_GraphicCtrl: GraphicCtrl port map ( -- play state machine
    ck          => ck24Hz,
    RESET       => RESET,
    hit_left    => hit_left,
    hit_right   => hit_right,
    start_game  => start_game,
    turbo_left  => turbo_left,
    turbo_right => turbo_right,
    led_state   => led_state,
    score_left  => score_left,
    score_right => score_right,
    lcd_config_string => lcd_config_string,
    lcd_message => lcd_message,
    lcd_mes_req => lcd_mes_req,
    lcd_mes_ack => lcd_mes_ack
    );

  Inst_LcdSwCtrl: LcdSwCtrl port map(  -- IO2 bus controller
    ck => ck3MHz,
    reset => reset,
    IO2_db => db,
--    IO2_adr =>DBadr ,
--    IO2_CS => cs,
--    IO2_OE => oe,
--    IO2_WE => we,
--    disp7s(15 downto 12) => disp7s(15 downto 12), 
--    disp7s(11 downto 8) => "1010",
        -- turns off Digit2 (in fact writes an "A"
--    disp7s(7 downto 4) => "1010",
        -- turns off Digit1 (in fact writes an "A"
--    disp7s(3 downto 0) => disp7s(3 downto 0),
--    LED_state => LED_state,
--    btn => btn,
--  sw => ,                            -- not used in this project
    Lcd_E => Lcd_E,
    Lcd_RS => Lcd_RS,
    Lcd_RW => Lcd_RW,
    Lcd_req => Lcd_req,
    Lcd_RS_req => Lcd_RS_req,
    Lcd_RW_req => Lcd_RW_req,
    Lcd_ack => Lcd_ack,
    Lcd_write_byte => Lcd_write_byte,
    Lcd_read_byte => Lcd_read_byte
    );

  disp7s(15 downto 12) <= conv_std_logic_vector(score_left,4);
  disp7s(3 downto 0) <= conv_std_logic_vector(score_right,4);

  Inst_lcd_mess: LCD_mess_Ctrl port map(           -- IO5 bus controller
    ck => ck3MHz,
    reset => reset,
    lcd_message => lcd_message,
    lcd_mes_req => lcd_mes_req,
    lcd_mes_ack => lcd_mes_ack,
    lcd_config_string  =>  lcd_config_string,
    lcd_RS_req => lcd_RS_req, -- LCD Register Select request('1' - data,
                              --                      '0' - instruction)
    lcd_RW_req => lcd_RW_req, -- LCD Read/Write request ('1' - read, 
                              --                         '0' - write)
    LCD_write_byte => LCD_write_byte,      -- the byte to send to LCD
    LCD_ack => LCD_ack,                    -- LCD cycle acknowledge
    LCD_req => LCD_req                     -- LCD cycle request
    );


-- assigning the used keys
  turbo_left  <= btn(1);   
  hit_left   <= btn(0);    
  turbo_right  <= btn(3); 
  hit_right  <= btn(2);   
  start_game  <= not btn(4);  

-- assigning CPLD clock (7 segment display refresh clock)
--  sSegCk <= ck1KHz;

end structural;
